/******************************************************************************
 * $Id$
 *
 * Project:  libLAS - http://liblas.org - A BSD library for LAS format data.
 * Purpose:  A composite class to handle read/write operations
 * Author:   Mateusz Loskot, mateusz@loskot.net
 *
 ******************************************************************************
 * Copyright (c) 2008, Mateusz Loskot
 * Copyright (c) 2008, Howard Butler
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following 
 * conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided 
 *       with the distribution.
 *     * Neither the name of the Martin Isenburg or Iowa Department 
 *       of Natural Resources nor the names of its contributors may be 
 *       used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 ****************************************************************************/

#ifndef LIBLAS_DETAIL_FILE_HPP_INCLUDED
#define LIBLAS_DETAIL_FILE_HPP_INCLUDED

#include <liblas/lasreader.hpp>
#include <liblas/laswriter.hpp>
#include <memory>
#include <string>
#include <fstream>
#include <iostream>
#include <stdexcept>
#include <cassert>

namespace liblas { namespace detail {

class FileImpl
{
public:

    FileImpl(std::string const& filename);
    FileImpl(std::string const& filename, LASHeader const& header, int mode);
    ~FileImpl();

    std::string GetName() const;
    int GetMode() const;
    LASHeader const& GetHeader() const;
    LASReader& GetReader();
    LASWriter& GetWriter();

private:

    // Blocked copying operations, declared but not defined.
    FileImpl(FileImpl const& other);
    FileImpl& operator=(FileImpl const& rhs);

    int m_mode;
    std::string m_filename;
    std::istream* m_istrm;
    std::ostream* m_ostrm;
    LASReader* m_reader;
    LASWriter* m_writer;
    LASHeader m_header;

    void throw_no_file_error() const;
};

}} // namespace liblas::detail

#endif // LIBLAS_DETAIL_FILE_HPP_INCLUDED
